function RUNcompare_sys()
    addpath(strcat(pwd(), '\Parameters'));
    addpath(strcat(pwd(), '\Trajectory'));
    addpath(strcat(pwd(), '\System'));
    
    init_parameters;
    init_trajectory('roundabout');
    
    global Q R S
    Q = 3500*diag([100,1,1,1,1]);
    F = Q;
    R = eye(3);
    
    fprintf('Solving Riccati equation\n');
    S = SolveRiccati1(Theta(5), Theta(0), F);

    th0 = Theta(0);
    Dth0 = DTheta(th0);
    
    endTime = 1;
    
    delta_x = [0, 0.002, 0, 0, 0, 0];
    x_initial = [phi1(th0), phi2(th0), phi3(th0), dphi1(th0)*Dth0, dphi2(th0)*Dth0, dphi3(th0)*Dth0] + delta_x;
    
    y_initial = ChangeVariables1(x_initial', th0);
    [theta, y] = integrate_tvl_sys(y_initial, 0, endTime);
    y = y';
    theta = theta';
    x = ChangeVariablesBack1(y, theta);
    
    H = figure(1);
    set(H, 'name', 'Simulation in path coordinate');
    subplot(211);
    hold on
    t = get_time(theta');
    plot(t, x(1,:), 'r');
    plot(t, x(2,:), 'g');
    plot(t, x(3,:), 'b');
    plot(t, x(4,:), 'm');
    plot(t, x(5,:), 'c');
    plot(t, x(6,:), 'y');
    xlabel('time');
    legend('q1', 'q2', 'q3', 'Dq1', 'Dq2', 'Dq3');
    subplot(212);
    hold on
    plot(theta,y(1,:), 'r');
    plot(theta,y(2,:), 'g');
    plot(theta,y(3,:), 'b');
    plot(theta,y(4,:), 'c');
    plot(theta,y(5,:), 'm');
    xlabel('theta');
    legend('I','y1','y2','Dy1', 'Dy2');  
    
%     
    
    [t2,x2] = integrate_time_sys(x_initial, 0, endTime);
    H = figure(2);
    set(H, 'name', 'Simulation in time coordinate');
    subplot(211);
    hold on 
    x2 = x2';
    plot(t2, x2(1,:), 'r');
    plot(t2, x2(2,:), 'g');
    plot(t2, x2(3,:), 'b');
    plot(t2, x2(4,:), 'm');
    plot(t2, x2(5,:), 'c');
    plot(t2, x2(6,:), 'y');
    xlabel('time');
    legend('q1', 'q2', 'q3', 'Dq1', 'Dq2', 'Dq3');   

    subplot(212);
    theta = invphi1(x2(1,:));
    y = ChangeVariables1(x2,theta);    
    hold on
    plot(theta,y(1,:), 'r');
    plot(theta,y(2,:), 'g');
    plot(theta,y(3,:), 'b');
    plot(theta,y(4,:), 'c');
    plot(theta,y(5,:), 'm');
    xlabel('theta');    
    legend('I','y1','y2','Dy1', 'Dy2');      
% 
    x1pp = pchip(t,x);
    x2pp = pchip(t2,x2);

    t = linspace(0,endTime,100);
    xdiff = ppval(x1pp, t) - ppval(x2pp, t);

    H = figure(3);
    set(H, 'name', 'Comparison of q coordinates');
    hold on
    plot(t, xdiff(1,:), 'r');
    plot(t, xdiff(2,:), 'g');
    plot(t, xdiff(3,:), 'b');
    plot(t, xdiff(4,:), 'm');
    plot(t, xdiff(5,:), 'c');
    plot(t, xdiff(6,:), 'y');
    legend('q1', 'q2', 'q3', 'Dq1', 'Dq2', 'Dq3'); 
    
end

function [t,x] = integrate_time_sys(x_initial, t_start, t_end)
    options = odeset('Events',@events, 'MaxStep',1e-3, 'AbsTol', 1e-6, 'RelTol', 1e-6);
    [t,x] = ode45(@fct_cl_system2,  [t_start, t_end], x_initial, options);
end


function [value,isterminal,direction] = events(t,y)
    value(1) = Theta(15) - y(1);
    isterminal = 1;
    direction = 0;
end

function dx = fct_cl_system2(t,x)
    th = invphi1(x(1),1);
    y = ChangeVariables1(x,th);
    
    v = fct_tl_control1(th, y);
    
    u = feedbacktransformation(x,v,th);
    dx = fct_without_load_sys(x,u);

end


function [theta,y] = integrate_tvl_sys(y_initial, TI, TE)
    options = odeset('MaxStep',1e-3, 'AbsTol', 1e-6, 'RelTol', 1e-6);
    [theta, y] = ode45(@fct_tv_control2, [Theta(TI),Theta(TE)], y_initial, options);    
end

function dy = fct_tv_control2(th,y)
    global Q R S
    A = Aaux_r1(th);
    B = Baux_r1(th);
    Ss = reshape(ppval(S, th),5,5);
    L = R\B'*Ss;
    v = -L*y;
    
    dy = A*y + B*v;
end

