function RUNcompare_sys2()
    addpath(strcat(pwd(), '\Parameters'));
    addpath(strcat(pwd(), '\Trajectory'));
    addpath(strcat(pwd(), '\System'));
    
    init_parameters;
    init_trajectory('roundabout');
    
    global Q R S endTime
    %Q = 80000*eye(5);
    Q = 80000*diag([0.1, 100, 100, 0.1, 0.1]);
    %Q = 80000*eye(5);
    F = Q;
    R = eye(3);
    
    fprintf('Solving Riccati equation\n');
    S = SolveRiccati1(Theta(5), Theta(0), F);

    th0 = Theta(0);
    Dth0 = DTheta(th0);
    
    
    delta_x = [0, -0.2, 0, 0, 0, 0];
    x_initial = [phi1(th0), phi2(th0), phi3(th0), dphi1(th0)*Dth0, dphi2(th0)*Dth0, dphi3(th0)*Dth0];
    x_initial = x_initial + delta_x;
    
     
    [t,x] = integrate_time_sys(x_initial, 0, endTime);
    fprintf('Integration took %f seconds\n', t(end))
    H = figure(2);
    set(H, 'name', 'Simulation in time coordinate');
    subplot(211);
    hold on 
    x = x';
    plot(t, x(1,:), 'r');
    plot(t, x(2,:), 'g');
    plot(t, x(3,:), 'b');
    plot(t, x(4,:), 'm');
    plot(t, x(5,:), 'c');
    plot(t, x(6,:), 'y');
    xlabel('time');
    legend('q1', 'q2', 'q3', 'Dq1', 'Dq2', 'Dq3');   

    subplot(212);
    theta = invphi1(x(1,:));
    y = ChangeVariables1(x,theta);    
    hold on
    plot(theta,y(1,:), 'r');
    plot(theta,y(2,:), 'g');
    plot(theta,y(3,:), 'b');
    plot(theta,y(4,:), 'c');
    plot(theta,y(5,:), 'm');
    xlabel('theta');    
    legend('I','y1','y2','Dy1', 'Dy2');      
   
    H = figure(66);
    c = [length(x), 1];
    plot_robot(x',c);
    
    H = figure(88);
    subplot(212);
    hold on
    
    speedDeviation = [];
    positionDeviation = [];
    for k = 1:length(y)
        speedDeviation = [speedDeviation, norm([y(1,k), y(4,k), y(5,k)])];
        positionDeviation = [positionDeviation, norm([y(2,k), y(3,k)])];
    end

    plot(theta, speedDeviation, 'r');
    plot(theta, positionDeviation, 'b');
    legend('Deviation in velocity', 'Deviation in position')
    title('System2')
end

function [t,x] = integrate_time_sys(x_initial, t_start, t_end)
    %options = odeset('Events',@events, 'MaxStep',1e-3, 'AbsTol', 1e-6, 'RelTol', 1e-6);
    options = odeset('Events',@events);
    [t,x] = ode45(@fct_cl_system2,  [t_start, t_end], x_initial, options);
end


function [value,isterminal,direction] = events(t,y)
    global endTime
    value(1) = Theta(endTime) - y(1);
    isterminal = 1;
    direction = 0;
end

function dx = fct_cl_system2(t,x)
    th = invphi1(x(1),1);
    fprintf('th: %f\n', th)
    y = ChangeVariables1(x,th);
    
    v = fct_tl_control1(th, y);
    u = feedbacktransformation(x,v,th);
    dx = fct_without_load_sys(x,u);

end
