function RUNcontrol_circle()
    addpath(strcat(pwd(), '\Parameters'));
    addpath(strcat(pwd(), '\Trajectory'));
    addpath(strcat(pwd(), '\System'));
    
    init_parameters;
    init_trajectory;
    
    global Q R S
    %Q = 3500*eye(5);
    Q = 3500*diag([0.1, 100, 100, 0.1, 0.1]);
    F = Q;
    R = eye(3);
    
    global Q1 R1 F1 Q2 R2 F2 Q3 R3 F3 Q4 R4 F4
    Q1 = Q; Q2 = Q; Q3 = Q; Q4 = Q;
    R1 = R; R2 = R; R3 = R; R4 = R;
    F1 = F; F2 = F; F3 = F; F4 = F;
    
    
    th0 = pi/2-0.6;
    %Dth0 = DTheta(get_time(th0));
    Dth0 = DTheta(th0);

    xdelta = [0.1,0.1,0,0,0,0];
    
    x0 = [phi1(th0), phi2(th0), phi3(th0), dphi1(th0)*Dth0, dphi2(th0)*Dth0, dphi3(th0)*Dth0];
    x0 = x0 + xdelta;
    
    global control_feedback_nominal control_feedback_deviation
    control_feedback_nominal = [];
    control_feedback_deviation = [];
    
    %[c,t,x] = integrate_system(0,  0.4809, x0);
    [c,t,x] = integrate_system(0, 10, x0);
    H = figure(3);
    plot_joints_and_deviations(c,t,x);

    H = figure(66);
    plot_robot(x,c);


end

function plot_joints_and_deviations(c,t,x)
    H = gcf();
    set(H,'name', 'Controlled by Transverse Linearization controller');
    subplot(211);
    hold on;plot(t,x(:,1), 'r');plot(t,x(:,2), 'g');plot(t,x(:,3), 'b');
    legend('q1','q2','q3');
    
    prev = [1,0];
    y = [];
    th = [];
    for i = 1:size(c,1)
        cc = c(i,:);
        tt = t(prev(1):cc(1));
        xx = x(prev(1):cc(1), 1:6);
        if cc(2) == 1
            yy = ChangeVariables1(xx',invphi1(xx(:,1),1)')';
        elseif cc(2) == 2
            yy = ChangeVariables3(xx',invphi3(xx(:,1),2)')';
        elseif cc(2) == 3
            yy = ChangeVariables1(xx',invphi1(xx(:,1),2)')';
        else
            yy = ChangeVariables3(xx',invphi3(xx(:,1),3)')';
        end
        theta = Theta(tt);
        th = [th;theta];
        y = [y;yy];
        prev = cc;
    end
    th = th';
    y = y';
    subplot(212);
    hold on
    plot(th,y(1,:), 'r');
    plot(th,y(2,:), 'g');
    plot(th,y(3,:), 'b');
    plot(th,y(4,:), 'c');
    plot(th,y(5,:), 'm');
    legend('I','y1','y2','Dy1', 'Dy2');  
    fprintf('\n');
end

function [c,t,x] = integrate_system(t_start, t_end, x0)
    %options = odeset('Events',@events, 'OutputFcn', @store_control_feedback);
    options1 = odeset('Events',@events1);
    options2 = odeset('Events',@events2);
    options3 = odeset('Events',@events3);
    options4 = odeset('Events',@events4);    
    
    t = [0]; x = [];
    c = [];
    
    fprintf('Integrate system\n')
    
    timeleft = t_end-t_start;
    
    cur_controller = 0;
    while(timeleft > 0)
        theta = calc_theta(x0(1), x0(2), x0(3));
        fprintf('Theta: %f    timeleft: %f\n', theta,timeleft)
        tt = []; xx = [];
        if theta >= 1/4*pi && theta < 3/4*pi
            fprintf('  Controller 1\n')
            [tt,xx] = ode45(@fct_circle1,  [0, timeleft], x0, options1);
            cur_controller = 1;
        elseif theta >= 3/4*pi && theta < 5/4*pi
            fprintf('  Controller 2\n')            
            [tt,xx] = ode45(@fct_circle2,  [0, timeleft], x0, options2);
            cur_controller = 2;
        elseif theta >= 5/4*pi && theta < 7/4*pi
            fprintf('  Controller 3\n')            
            [tt,xx] = ode45(@fct_circle3,  [0, timeleft], x0, options3);
            cur_controller = 3;
        elseif (theta >= 7/4*pi && theta < 8/4*pi) || (theta >= 0 && theta < 1/4*pi)
            fprintf('  Controller 4\n')            
            [tt,xx] = ode45(@fct_circle4,  [0, timeleft], x0, options4);
            cur_controller = 4;
        else
            disp('Unsupported theta');
            return;
        end
        t = [t; t(end)+tt];
        x = [x;xx];
        c = [c; length(x), cur_controller];            
        timeleft = timeleft - tt(end);
        x0  = xx(end,:);
    end
    t = t(2:end);
    theta = calc_theta(x0(1), x0(2), x0(3));
    fprintf('Theta: %f    timeleft: %f\n', theta,timeleft)
end

function theta = calc_theta(q1,q2,q3)
    global ricxc ricyc riczc ricd1 rica1 rica2 rica3 
    endeffector = [(cos(q1) .* cos(q2) - sin(q1) .* cos(pi ./ 0.2e1) .* sin(q2)) .* rica3 .* cos(q3) + (-cos(q1) .* sin(q2) - sin(q1) .* cos(pi ./ 0.2e1) .* cos(q2)) .* rica3 .* sin(q3) + cos(q1) .* rica2 .* cos(q2) - sin(q1) .* cos(pi ./ 0.2e1) .* rica2 .* sin(q2) + cos(q1) .* rica1 (sin(q1) .* cos(q2) + cos(q1) .* cos(pi ./ 0.2e1) .* sin(q2)) .* rica3 .* cos(q3) + (-sin(q1) .* sin(q2) + cos(q1) .* cos(pi ./ 0.2e1) .* cos(q2)) .* rica3 .* sin(q3) + sin(q1) .* rica2 .* cos(q2) + cos(q1) .* cos(pi ./ 0.2e1) .* rica2 .* sin(q2) + sin(q1) .* rica1 -sin(pi ./ 0.2e1) .* sin(q2) .* rica3 .* cos(q3) - sin(pi ./ 0.2e1) .* cos(q2) .* rica3 .* sin(q3) - sin(pi ./ 0.2e1) .* rica2 .* sin(q2) + ricd1];
    delta = endeffector-[ricxc,ricyc,riczc];
    theta = atan2(delta(3), delta(2));
    if theta < 0
        theta = 2*pi + theta;
    end
end

function [value,isterminal,direction] = events1(t,y)
    % Locate the time when height passes through zero in a decreasing direction
    % and stop integration.
    value(1) = 3/4*pi - invphi1(y(1),1);
    value(2) = 3/4*pi+0.1 - invphi1(y(1),1);
    isterminal = [1,1];
    direction = [0,0];
end

function [value,isterminal,direction] = events2(t,y)
    % Locate the time when height passes through zero in a decreasing direction
    % and stop integration.
    value(1) = 5/4*pi - invphi3(y(3),2);
    value(2) = 5/4*pi+0.1 - invphi3(y(3),2);    
    isterminal = [1,1];
    direction = [0,0];
end

function [value,isterminal,direction] = events3(t,y)
    % Locate the time when height passes through zero in a decreasing direction
    % and stop integration.
    value(1) = 7/4*pi - invphi1(y(1),2);
    value(2) = 7/4*pi+0.1 - invphi1(y(1),2);    
    isterminal = [1,1];
    direction = [0,0];
end

function [value,isterminal,direction] = events4(t,y)
    % Locate the time when height passes through zero in a decreasing direction
    % and stop integration.
    value(1) = 9/4*pi - invphi3(y(3),3);
    value(2) = 9/4*pi+0.1 - invphi3(y(3),3);    
    value(3) = 9/4*pi+0.5 - invphi3(y(3),3);
    isterminal = [1,1,1];
    direction = [0,0,0];
end
