function RUNcontrol_inverse_dynamics()

    addpath(strcat(pwd(), '\Parameters'));
    addpath(strcat(pwd(), '\Trajectory'));
    addpath(strcat(pwd(), '\System'));
    
    init_parameters;
    init_trajectory('roundabout');
    
    global Kp Kd endTime
    Kp = 10*diag([1,1,1]);
    Kd = 2*diag([1,1,1]);

    th0 = 0;
    Dth0 = DTheta(get_time(th0));
    
    tStart = get_time(th0);
    tEnd = endTime;
    
    delta_x = [0, -0.2, 0, 0, 0, 0];
    x0 = [phi1(th0), phi2(th0), phi3(th0), dphi1(th0)*Dth0, dphi2(th0)*Dth0, dphi3(th0)*Dth0];
    x0 = x0 + delta_x;
    
    global control_feedback
    control_feedback = [];
    
    [t,x] = integrate_system(tStart, tEnd, x0);

    H = figure(1);
    hold on;plot(t,x(:,1), 'r');plot(t,x(:,2), 'g');plot(t,x(:,3), 'b');
    %plot(t,x(:,4), 'c');plot(t,x(:,5), 'm');plot(t,x(:,6), 'y');
    set(H,'name', 'Controlled by Inverse Dynamics controller');
    legend('q1','q2','q3','Dq1','Dq2','Dq3');
    fprintf('\n');
    plot_control_feedback(control_feedback);

    H = figure(3);
    x = x';
    y = ChangeVariables1(x,x(1,:));    
    hold on
    plot(x(1,:),y(1,:), 'r');
    plot(x(1,:),y(2,:), 'g');
    plot(x(1,:),y(3,:), 'b');
    plot(x(1,:),y(4,:), 'c');
    plot(x(1,:),y(5,:), 'm');
    xlabel('theta');    
    legend('I','y1','y2','Dy1', 'Dy2');     
    x = x';
    H = figure(7);
    plot_robot(x);
end

function plot_control_feedback(control_feedback)
    global endTime
    control_feedback = sortrows(control_feedback',1)';
    [b,i,j] = unique(control_feedback(1,:));
    
    cfpp = interp1(control_feedback(1,i), control_feedback(2:end,i)', 'pchip', 'pp');
    H = figure(2);
    t = linspace(0,endTime,100);
    control_feedback = ppval(cfpp, t)';
    hold on;
    plot(t, control_feedback(1,:), 'r');
    plot(t, control_feedback(2,:), 'g');
    plot(t, control_feedback(3,:), 'b');    
    legend('u1', 'u2', 'u3');
    set(H, 'name', 'Control given by Inverse Dynamics');
end

function [t,x] = integrate_system(tStart, tEnd, x0)
    options = odeset('Events',@events, 'OutputFcn', @store_control_feedback);
     [t,x] = ode45(@fct_cl_system, [tStart, tEnd], x0, options);
     
%     fprintf('Integrate 0,5\n')
%     [t1,x1] = ode45(@fct_cl_system,  [tStart, 10], x0, options);
%     fprintf('Integrate 5,5.2\n')
%     [t2,x2] = ode45(@fct_cl_system2, [10, 10.03], x1(end,:), options);    
%     fprintf('Integrate 5.2,15\n')
%     [t3,x3] = ode45(@fct_cl_system,  [10.03, tEnd], x2(end,:), options);    
%     t = [t1;t2;t3];
%     x = [x1;x2;x3];    
end

function status = store_control_feedback(tt,x,flag)
    if strcmp(flag, 'init')
        return;
    end
    
    global control_feedback
    for i = 1:length(tt)
        t = tt(i);
        u = fct_id_control(t,x(:,i));
        control_feedback = [control_feedback,[t;u]];
    end
    status = 0;
end


function dx = fct_cl_system2(t,x)
    dx = fct_without_load_sys(x,[0,0,0]');
end

function dx = fct_cl_system(t,x)
    u = fct_id_control(t,x);
    dx = fct_without_load_sys(x,u);
end

function [value,isterminal,direction] = events(t,y)
    % Locate the time when height passes through zero in a decreasing direction
    % and stop integration.
    value = Theta(14.4) - y(1);
    isterminal = 1;   % stop the integration
    direction = -1;   % negative direction
end

function v = fct_id_control(t,x)
    global Kp Kd
    
    dt = desired_trajectory(t);    
    aq = dt(7:9)' - Kp*(x(1:3) - dt(1:3)') - Kd*(x(4:6) - dt(4:6)');
    M = Mass_without_load(x(1:3));
    C = Coriolis_without_load(x(1:3), x(4:6));
    G = Gravity_without_load(x(1:3));
    
    v = C*x(4:6) + G + M*aq;
end

function d = desired_trajectory(t)
    th   = Theta(t);
    Dth  = DTheta(th);
    DDth = DDTheta(th);
    
    d(1) = phi1(th);
    d(2) = phi2(th);
    d(3) = phi3(th);
    
    d(4) = dphi1(th)*Dth;
    d(5) = dphi2(th)*Dth;
    d(6) = dphi3(th)*Dth;
    
    d(7) = ddphi1(th).*Dth.^2 + dphi1(th).*DDth;
    d(8) = ddphi2(th).*Dth.^2 + dphi2(th).*DDth;
    d(9) = ddphi3(th).*Dth.^2 + dphi3(th).*DDth;    
end