close all;

addpath(strcat(pwd(), '\path'));


% Number of grid points.
sgrid = 1000;

taumax = [500;400;100];

% Lower and upper bounds for the torques.
torquelb = -taumax*0.8;
torqueub = taumax*0.8;
nrdof = size(torquelb,1);



% The path function.
pathfun = @circlepath;
% The manipulator dynamics.
invdynfun = @invdyn;

% Set to 1 to allow Coulomb friction in the dynamic model.
coulomb = 0;
% Fetch all path-dependent quantities.
[s, sm, ds, q, qp, qpp, Mv, Cv, gv] = dynpathparams(invdynfun, pathfun, sgrid, coulomb);

[a, b, c, d, tau, diagnostics] = timeoptpathconstrsed(s, sm, ds, q, qp, qpp, Mv, Cv, gv, torquelb, torqueub, 0, 0, 'sedumi');
% ---------------------------------------

t = zeros(1,length(s)-1);
% Obtain the time vector.
t = cumsum(2*ds./(sqrt(b(2:end))+sqrt(b(1:end-1))));
%[xee,yee,zee] = fwkinkuka361(q);
bm = (b(2:end)+b(1:end-1))/2;
qdot = repmat(sqrt(bm),3,1).*qp;
qddot = repmat(a,3,1).*qp + repmat(bm,3,1).*qpp;
H = figure(1);
%subplot(3,1,1);
plot(t, q(1,:));
hold on
plot(t, q(2,:), 'g');
plot(t, q(3,:), 'r');
legend('q1', 'q2', 'q3');
set(H, 'name', 'Path variables');
H = figure(2);
%subplot(3,1,2);
plot(t, qdot(1,:));
hold on
plot(t, qdot(2,:),'g');
plot(t, qdot(3,:), 'r');
legend('Dq1', 'Dq2', 'Dq3');
set(H, 'name', 'Velocity variables');
H = figure(3);
%subplot(3,1,3);
plot(t, qddot(1,:));
hold on
plot(t, qddot(2,:),'g');
plot(t, qddot(3,:),'r');
legend('DDq1', 'DDq2', 'DDq3');
set(H, 'name', 'Acceleration variables');

mot = [interp1(t,q(1,:),linspace(0,max(t), 100), 'pchip')', interp1(t,q(2,:),linspace(0,max(t), 100), 'pchip')', interp1(t,q(3,:),linspace(0,max(t), 100), 'pchip')'];

% This is the integral of the torques squared
%tausqs = sum(sum(repmat(2*ds./(sqrt(b(2:end))+sqrt(b(1:end-1))),3,1).*(tau./repmat(torqueub,1,length(s)-1)).^2));
H = figure(4);
plot(t,tau(1,:),'b',t,tau(2,:),'g-.',t,tau(3,:),'k--')
set(H, 'name', 'Joint torques')
legend('q1','q2','q3','location','SouthEast');
as = axis;
axis([as(1) as(2) -1.1*max(torqueub) 1.1*max(torqueub)]);

