function pointlist()
    number_of_points = 8;
    point_list = [];
    
    for i = 1:number_of_points
        point_list = [point_list;
            [rand()/number_of_points, rand()/number_of_points, rand()/number_of_points]];
    end
    
    point_list = cumsum(point_list);
    
    H = figure(1);
    set(H, 'name', 'Spline parameterization');
    [splineBefore, splineAfter, splineTime, splineDeltaX] = do_optimize(point_list, 'spline');
    H = figure(2);
    set(H, 'name', 'PCHIP parameterization');
    [pchipBefore, pchipAfter, pchipTime, pchipDeltaX] = do_optimize(point_list, 'pchip');
    
    H = figure(3);
    set(H, 'name', 'Spline velocity and acceleration');
    plot_velocities(splineDeltaX, 'spline');
    
    H = figure(4);
    set(H, 'name', 'PCHIP velocity and acceleration');
    plot_velocities(pchipDeltaX, 'pchip');
    
    fprintf('Spline parameterization in %f seconds. Before: %f   After: %f\n', splineTime, splineBefore, splineAfter);
    fprintf('PCHIP parameterization. in %f seconds. Before: %f   After: %f\n', pchipTime, pchipBefore, pchipAfter);    
    
end

function plot_velocities(deltaX, spline_type)
    global type 
    type = spline_type;

    taumax = [500;400;100];
    % Lower and upper bounds for the torques.
    torquelb = -taumax*0.8;
    torqueub = taumax*0.8;
    
    % The weighting.
    gamma1 = 0;
    gamma2 = 0;
    
    % Set to 1 to allow Coulomb friction in the dynamic model.
    coulomb = 0;
    
    sgrid = 1000;
    
    global deltaPositions number_of_points
    number_of_points = size(deltaX, 1);
    
    deltaPositions = deltaX;
    
    % Fetch all path-dependent quantities.
    [s, sm, ds, q, qp, qpp, Mv, Cv, gv] = dynpathparams(@invdyn, @pathfunction, sgrid, coulomb);
    [a, b, c, d, tau, diagnostics] = timeoptpathconstrsed(s, sm, ds, q, qp, qpp, Mv, Cv, gv, torquelb, torqueub, gamma1(1), gamma2(1), 'sedumi');
    
	t = zeros(1,length(s)-1);
	% Obtain the time vector.
	t = cumsum(2*ds./(sqrt(b(2:end))+sqrt(b(1:end-1))));
	%[xee,yee,zee] = fwkinkuka361(q);
	bm = (b(2:end)+b(1:end-1))/2;
	qdot = repmat(sqrt(bm),3,1).*qp;
	qddot = repmat(a,3,1).*qp + repmat(bm,3,1).*qpp;

    %subplot(3,1,1);
%     plot(t, q(1,:));
%     hold on
%     plot(t, q(2,:), 'g');
%     plot(t, q(3,:), 'r');
%     legend('q1', 'q2', 'q3');
%     set(H, 'name', 'Path variables');
%     H = figure(2);
    subplot(2,1,1);
    plot(t, qdot(1,:));
    hold on
    plot(t, qdot(2,:),'g');
    plot(t, qdot(3,:), 'r');
    legend('Dq1', 'Dq2', 'Dq3');
    title('Velocity variables');
%     H = figure(3);
    subplot(2,1,2);
    plot(t, qddot(1,:));
    hold on
    plot(t, qddot(2,:),'g');
    plot(t, qddot(3,:),'r');
    legend('DDq1', 'DDq2', 'DDq3');
    title('Acceleration variables');
end

function [tBefore, tAfter, Time, deltaX] = do_optimize(list_of_points, spline_type)

    global type
    type = spline_type;

    global number_of_points
    number_of_points = size(list_of_points,1);
    
    global point_list
    point_list = list_of_points;
    
    plot3(point_list(:,1), point_list(:,2), point_list(:,3), 'black')
    
    xlabel('q1');
    ylabel('q2');
    zlabel('q3');
    
    pp = interp1(((1:number_of_points)-1)/(number_of_points-1), point_list, type, 'pp');
    t = linspace(0,1,100);
    xyz = ppval(pp, t);
    
    global original_pp
    original_pp = pp;
    
    hold on
    plot3(xyz(:,1), xyz(:,2), xyz(:,3), 'b');
    
    % calculate normals
    
    n = point_list(1,:) - point_list(2,:);
    n = n / norm(n);
    normals = n;
    
    for i = 2:(number_of_points-1)
        n = point_list(i-1,:) - point_list(i+1,:);
        n = n / norm(n);
        normals(i,:) = n;
    end
    
    n = point_list(end-1,:) - point_list(end,:);
    n = n / norm(n);
    normals = [normals; n];
    
    
    % Plot circles
    global radiuses
    radiuses = ones(number_of_points,1)/30;
    
    t = linspace(0,2*pi, 20);
    circx = cos(t);
    circy = sin(t);
    
    for i = 1:number_of_points
        [bv1, bv2] = find_new_basis(normals(i,:)');
        p = radiuses(i)*kron(bv1, circx) + radiuses(i)*kron(bv2, circy) + kron(point_list(i,:)', ones(1,length(t)));
        plot3(p(1,:), p(2,:), p(3,:), 'r')
    end
    
    global Aeq beq
    
    % Build matrices for optimization
    Aeq = [];
    for i = 2:(number_of_points-1)
        Aeq = blkdiag(Aeq, normals(i,:));
    end
    beq = zeros(number_of_points-2,1);
    
    % Dont allow start and end points to move
    Aeq = blkdiag(eye(3), Aeq);
    Aeq = blkdiag(Aeq, eye(3));
    beq = [[0;0;0]; beq; [0;0;0]];
    
    x0 = zeros(3*number_of_points,1);
    
    tstart = tic;
    deltaX = fmincon(@objectivefunction, x0, [], [], Aeq, beq, [], [], @nonlincon);
    Time = toc(tstart);
    
    tBefore = objectivefunction(x0, 1000);
    tAfter = objectivefunction(deltaX, 1000);
    
    deltaX = reshape(deltaX, 3, number_of_points)';
    
    points = point_list + deltaX;
    
    pp = interp1(((1:number_of_points)-1)/(number_of_points-1), points, type, 'pp');
    t = linspace(0,1,100);
    xyz = ppval(pp, t);
    
    hold on
    plot3(xyz(:,1), xyz(:,2), xyz(:,3), 'r', 'LineWidth', 2);
    
end

% For 100 points: 0.7937
% For 200 points: 0.8376s
% For 500 points: 0.8621

% deltaX500 = [
%          0         0         0;
%     0.0289    0.0104   -0.0129;
%     0.0238    0.0052   -0.0228;
%    -0.0050    0.0120   -0.0011;
%    -0.0041    0.0271   -0.0190;
%    -0.0165    0.0279   -0.0076;
%    -0.0162    0.0143    0.0023;
%          0         0         0];
%
% deltaX100 =[
%          0         0         0;
%     0.0289    0.0105   -0.0129;
%     0.0185   -0.0167   -0.0091;
%    -0.0210    0.0231    0.0117;
%    -0.0238    0.0199    0.0088;
%    -0.0208    0.0053    0.0141;
%    -0.0106    0.0076    0.0028;
%          0         0         0];