% function [pp] = mos(fs, dfs, fm, dfm, ddfm, fe, dfe)
% Create a two piece C^2 spline on the interval [0,1] given the following:
% fs   - f_start
% fe   - f_end
% fm   - f_midpoint
% dfs  - f'_start
% dfe  - f'_end
% dfm  - f'_midpoint
% ddfm - f''_midpoint
function [pp, dpp, ddpp] = mos(fs, dfs, fm, dfm, ddfm, fe, dfe)
% Here we want to determine f''(1)
m = 0.5;

A_start = [0 0 0 0 1; m^4 m^3 m^2 m 1; 0 0 0 1 0; 4*m^3 3*m^2 2*m 1 0; 12*m^2 6*m 2 0 0];

% Here we want to determine f''(0)
A_end   = [0 0 0 0 1; m^4 m^3 m^2 m 1; 0 0 0 1 0; 4*m^3 3*m^2 2*m 1 0; 0 0 2 0 0];

v1 = A_start\[fs; fm; dfs; dfm; ddfm];
v2 = A_end  \[fm; fe; dfm; dfe; ddfm];

coefs  = [v1'; v2'];
breaks = [0,0.5,1];

pp = mkpp(breaks, coefs);

dpp = pp;
dpp.coefs(:,1) = 0;
dpp.coefs(:,2) = pp.coefs(:,1)*4;
dpp.coefs(:,3) = pp.coefs(:,2)*3;
dpp.coefs(:,4) = pp.coefs(:,3)*2;
dpp.coefs(:,5) = pp.coefs(:,4);

ddpp = dpp;
ddpp.coefs(:,1) = 0;
ddpp.coefs(:,2) = dpp.coefs(:,1)*4;
ddpp.coefs(:,3) = dpp.coefs(:,2)*3;
ddpp.coefs(:,4) = dpp.coefs(:,3)*2;
ddpp.coefs(:,5) = dpp.coefs(:,4);