function [q, qp, qpp] = circlepath(s)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  % Required kinematic robot parameters
  ricd1 = 352;
  rica1 = 70;
  rica2 = 360;
  rica3 = 445;

  % Parameters of circle
  ricR  = 100;
  ricxc = 450;
  ricyc = 0;
  riczc = 450;

  k = pi/4;
  a = 0.5;
  p = 3;
  p2 = 0.5;

  for i=1:length(s)
    q(i,:)   = [q1(2*pi*s(i));   q2(2*pi*s(i));   q3(2*pi*s(i))];
    qp(i,:)  = [dq1(2*pi*s(i));  dq2(2*pi*s(i));  dq3(2*pi*s(i))];
    qpp(i,:) = [ddq1(2*pi*s(i)); ddq2(2*pi*s(i)); ddq3(2*pi*s(i))];
  end

  q = q';
  qp = qp';
  qpp = qpp';
end

function val = q1(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = asin(ricR * cos(theta) / ricxc);
end
 
function val = dq1(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = -ricR * sin(theta) / ricxc * (0.1e1 - ricR ^ 2 * cos(theta) ^ 2 / ricxc ^ 2) ^ (-0.1e1 / 0.2e1);
end

function val = ddq1(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = -ricR * cos(theta) / ricxc * (0.1e1 - ricR ^ 2 * cos(theta) ^ 2 / ricxc ^ 2) ^ (-0.1e1 / 0.2e1) + ricR ^ 3 * sin(theta) ^ 2 / ricxc ^ 3 * (0.1e1 - ricR ^ 2 * cos(theta) ^ 2 / ricxc ^ 2) ^ (-0.3e1 / 0.2e1) * cos(theta);
end

function val = q2(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = -asin(sqrt(0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * rica3 / 0.2e1) - atan2(ricR * sin(theta) + riczc - ricd1, sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1);
end

function val = dq2(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = 0.2e1 * (ricR * sin(theta) * (-sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) + rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) + ricR * sin(theta) + riczc - ricd1) * cos(theta) * ricR * (((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2 - 0.2e1 * sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * rica1 + ricR ^ 2 * sin(theta) ^ 2 - 0.2e1 * ricR * (-riczc + ricd1) * sin(theta) - rica3 ^ 2 + rica1 ^ 2 - (ricd1 - riczc + rica2) * (-ricd1 + riczc + rica2)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + sqrt(0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.3e1 / 0.2e1) * rica3 ^ 2 * rica2 ^ 2 / 0.2e1) / rica3 / rica2 ^ 2 * (0.4e1 - (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) * rica3 ^ 2) ^ (-0.1e1 / 0.2e1) - (ricR * cos(theta) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) + (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) / (0.1e1 + (ricR * sin(theta) + riczc - ricd1) ^ 2 / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2);
end

function val = ddq2(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = 0.2e1 * (ricR * cos(theta) * (-sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) + rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) + ricR ^ 3 * sin(theta) ^ 2 / (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * cos(theta) + ricR ^ 3 * sin(theta) ^ 2 * (-sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) + rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.3e1 / 0.2e1) * cos(theta) + ricR * cos(theta)) * cos(theta) * ricR * (((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2 - 0.2e1 * sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * rica1 + ricR ^ 2 * sin(theta) ^ 2 - 0.2e1 * ricR * (-riczc + ricd1) * sin(theta) - rica3 ^ 2 + rica1 ^ 2 - (ricd1 - riczc + rica2) * (-ricd1 + riczc + rica2)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + sqrt(0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.3e1 / 0.2e1) * rica3 ^ 2 * rica2 ^ 2 / 0.2e1) / rica3 / rica2 ^ 2 * (0.4e1 - (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) * rica3 ^ 2) ^ (-0.1e1 / 0.2e1) - 0.2e1 * (ricR * sin(theta) * (-sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) + rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) + ricR * sin(theta) + riczc - ricd1) * sin(theta) * ricR * (((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2 - 0.2e1 * sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * rica1 + ricR ^ 2 * sin(theta) ^ 2 - 0.2e1 * ricR * (-riczc + ricd1) * sin(theta) - rica3 ^ 2 + rica1 ^ 2 - (ricd1 - riczc + rica2) * (-ricd1 + riczc + rica2)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + sqrt(0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.3e1 / 0.2e1) * rica3 ^ 2 * rica2 ^ 2 / 0.2e1) / rica3 / rica2 ^ 2 * (0.4e1 - (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) * rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + 0.2e1 * (ricR * sin(theta) * (-sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) + rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) + ricR * sin(theta) + riczc - ricd1) * cos(theta) * ricR * (-((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.3e1 / 0.2e1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2 - 0.2e1 * sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * rica1 + ricR ^ 2 * sin(theta) ^ 2 - 0.2e1 * ricR * (-riczc + ricd1) * sin(theta) - rica3 ^ 2 + rica1 ^ 2 - (ricd1 - riczc + rica2) * (-ricd1 + riczc + rica2)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) * (0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) - 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) / 0.2e1 + ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * (0.2e1 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * rica1 * ricR ^ 2 * cos(theta) * sin(theta) - 0.2e1 * ricR * (-riczc + ricd1) * cos(theta)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2 - 0.2e1 * sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * rica1 + ricR ^ 2 * sin(theta) ^ 2 - 0.2e1 * ricR * (-riczc + ricd1) * sin(theta) - rica3 ^ 2 + rica1 ^ 2 - (ricd1 - riczc + rica2) * (-ricd1 + riczc + rica2)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.3e1 / 0.2e1) * (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) / rica2 ^ 2 / rica3 ^ 2 * (-0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) - (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.3e1 / 0.2e1) * (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) * (-0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) / 0.2e1 - 0.3e1 / 0.4e1 * sqrt(0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.5e1 / 0.2e1) * rica3 ^ 2 * rica2 ^ 2 * (0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) - 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta))) / rica3 / rica2 ^ 2 * (0.4e1 - (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) * rica3 ^ 2) ^ (-0.1e1 / 0.2e1) - (ricR * sin(theta) * (-sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) + rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) + ricR * sin(theta) + riczc - ricd1) * cos(theta) * ricR * (((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.1e1 / 0.2e1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2 - 0.2e1 * sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * rica1 + ricR ^ 2 * sin(theta) ^ 2 - 0.2e1 * ricR * (-riczc + ricd1) * sin(theta) - rica3 ^ 2 + rica1 ^ 2 - (ricd1 - riczc + rica2) * (-ricd1 + riczc + rica2)) * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + sqrt(0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) * ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ (-0.3e1 / 0.2e1) * rica3 ^ 2 * rica2 ^ 2 / 0.2e1) / rica3 / rica2 ^ 2 * (0.4e1 - (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) * rica3 ^ 2) ^ (-0.3e1 / 0.2e1) * (0.2e1 * (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) / rica2 ^ 2 * (-0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) + (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) / ((ricR * sin(theta) + riczc - ricd1) ^ 2 + (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 * rica3 ^ 2 * (0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) - 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta))) - (-ricR * sin(theta) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) + 0.2e1 * ricR ^ 3 * cos(theta) ^ 2 / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * sin(theta) + 0.2e1 * (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 3 / (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * ricR ^ 4 * cos(theta) ^ 2 * sin(theta) ^ 2 + (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.3e1 / 0.2e1) * ricR ^ 4 * cos(theta) ^ 2 * sin(theta) ^ 2 - (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * sin(theta) ^ 2 + (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) ^ 2) / (0.1e1 + (ricR * sin(theta) + riczc - ricd1) ^ 2 / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) + (ricR * cos(theta) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) + (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) / (0.1e1 + (ricR * sin(theta) + riczc - ricd1) ^ 2 / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 * (0.2e1 * (ricR * sin(theta) + riczc - ricd1) / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2 * ricR * cos(theta) + 0.2e1 * (ricR * sin(theta) + riczc - ricd1) ^ 2 / (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 3 * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta));
end

function val = q3(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = pi - acos((rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) / rica2 / rica3 / 0.2e1);
end

function val = dq3(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = (-0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) / rica2 / rica3 * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1);
end

function val = ddq3(theta)
  global ricR ricxc ricyc riczc ricd1 rica1 rica2 rica3
  val = (-0.2e1 * ricR ^ 2 * cos(theta) ^ 2 + 0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * sin(theta) - 0.2e1 / (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) * ricR ^ 4 * cos(theta) ^ 2 * sin(theta) ^ 2 + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.3e1 / 0.2e1) * ricR ^ 4 * cos(theta) ^ 2 * sin(theta) ^ 2 - 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * sin(theta) ^ 2 + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) ^ 2) / rica2 / rica3 * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.1e1 / 0.2e1) + (-0.2e1 * (ricR * sin(theta) + riczc - ricd1) * ricR * cos(theta) + 0.2e1 * (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) * (ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) ^ (-0.1e1 / 0.2e1) * ricR ^ 2 * cos(theta) * sin(theta)) ^ 2 / rica2 ^ 3 / rica3 ^ 3 * (0.4e1 - (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2) ^ 2 / rica2 ^ 2 / rica3 ^ 2) ^ (-0.3e1 / 0.2e1) * (rica2 ^ 2 + rica3 ^ 2 - (ricR * sin(theta) + riczc - ricd1) ^ 2 - (sqrt(ricR ^ 2 * cos(theta) ^ 2 + ricxc ^ 2) - rica1) ^ 2);
end
