close all;
global savefigs;
% When set to 1, this will save all images in "./images/" directory.
savefigs = 0;

global invdynfun pathfun sgrid coulomb
global torquelb torqueub gamma1 gamma2
    
% Number of grid points.
sgrid = 200;

taumax = [500;400;100];

% Lower and upper bounds for the torques.
torquelb = -taumax*0.8;
torqueub = taumax*0.8;
nrdof = size(torquelb,1);

% The weighting.
gamma1 = 0;
gamma2 = 0;

% If gamma1 or gamma2 are vectors, this vector contains the trajectory times
% as a function of gamma1 or gamma2.
ts = zeros(1,max(length(gamma1),length(gamma2)));
% If gamma1 is a vector, this vector contains the integral of the squares of the torques
% or if gamma2 is a vector, this vector contains the integral of the absolute value
% of the rate of change of the torques.
tausqs = zeros(1,max(length(gamma1),length(gamma2)));

% Cell arrays to save all results if gamma1 or gamma2 are vectors.
tc = cell(1,max(length(gamma1),length(gamma2)));
sc = cell(1,max(length(gamma1),length(gamma2)));
smc = cell(1,max(length(gamma1),length(gamma2)));
tauc = cell(1,max(length(gamma1),length(gamma2)));
qc = cell(1,max(length(gamma1),length(gamma2)));
qdotc = cell(1,max(length(gamma1),length(gamma2)));
qddotc = cell(1,max(length(gamma1),length(gamma2)));
xeec = cell(1,max(length(gamma1),length(gamma2)));
yeec = cell(1,max(length(gamma1),length(gamma2)));
zeec = cell(1,max(length(gamma1),length(gamma2)));
bc = cell(1,max(length(gamma1),length(gamma2)));
ac = cell(1,max(length(gamma1),length(gamma2)));



% The path function.
pathfun = @varyingpath;
% The manipulator dynamics.
invdynfun = @invdyn;

% Set to 1 to allow Coulomb friction in the dynamic model.
coulomb = 0;	


f1m_vector=linspace(0,2,10);
df1m_vector=linspace(-3,1,15);

for i=1:length(f1m_vector)
    for j=1:length(df1m_vector)
        f1m = f1m_vector(i);
        df1m = df1m_vector(i);
        
        params = [f1m, df1m, 0, 3, 2, 0];
        Z(i,j) = objectivefunction(params);
    end
end

close all
figure(1);

[X,Y] = meshgrid(f1m_vector, df1m_vector);
surf(X,Y,Z');
xlabel('f1m');
ylabel('df1m');
