#include <math.h>
#include <QGLWidget>

#include "camera.h"

Camera::Camera() : rotation(1.0, 0.0, 0.0, 0.0)
{
  pos = MSLVector(3);

  forward = MSLVector(3);
  forward[1] = 1;

  view = forward;

  up = MSLVector(3);
  up[2] = 1;
}

void Camera::move(float d)
{
  pos = pos + view * d;
}

void Camera::Tilt(float t)
{
  MSLVector x(3);
  
  // Cross product
  x[0] =  up[1]*view[2] - up[2]*view[1];
  x[1] = -up[0]*view[2] + up[2]*view[0];
  x[2] =  up[0]*view[1] - up[1]*view[0];

  Quaternion pitch = Quaternion::axisAngle(t, -1, 0, 0);
  // Composition of rotations
  rotation = rotation * pitch;  
  rotation = rotation / rotation.norm();

  applyRotation();
}

void Camera::Yaw(float y)
{
  MSLVector x(3);
  // Cross product
  x[0] =  up[1]*view[2] - up[2]*view[1];
  x[1] = -up[0]*view[2] + up[2]*view[0];
  x[2] =  up[0]*view[1] - up[1]*view[0];

  Quaternion yaw = Quaternion::axisAngle(y, 0.0, 0.0, 1.0);
  // Composition of rotations
  rotation = rotation * yaw;
  rotation = rotation / rotation.norm();
}

void Camera::applyRotation()
{
  // Calculate new up and view direction vectors based on current rotation

  Quaternion u = rotation.rotate(Quaternion(0, 0, 0, 1));
  Quaternion v = rotation.rotate(Quaternion(0, 0, 1, 0));

  up[0] = u.b;
  up[1] = u.c;
  up[2] = u.d;

  up = up.norm();

  view[0] = v.b;
  view[1] = v.c;
  view[2] = v.d;

  view = view.norm();
}

void Camera::use()
{
  gluLookAt(pos[0], pos[1], pos[2], 
	    pos[0]+view[0], pos[1]+view[1], pos[2]+view[2], 
	    up[0], up[1], up[2]);
}
