#include "controls.h"

#ifndef PI
#define PI 3.14157f
#endif

Controls::Controls(Camera *camera, QWidget *parent) : QGroupBox(parent)
{
  this->camera = camera;
  layout = new QFormLayout;
  setLayout(layout);
  
  QLabel *xpos = new QLabel("xpos");
  QLabel *ypos = new QLabel("ypos");
  QLabel *zpos = new QLabel("zpos");
  xposSpinBox = new QSpinBox;
  yposSpinBox = new QSpinBox;
  zposSpinBox = new QSpinBox;

  xposSpinBox->setRange(-1000, 1000);
  xposSpinBox->setSingleStep(10);
  yposSpinBox->setRange(-1000, 1000);
  yposSpinBox->setSingleStep(10);
  yposSpinBox->setValue(-1000.0);
  zposSpinBox->setRange(-1000, 1000);
  zposSpinBox->setValue(500);
  zposSpinBox->setSingleStep(10);

  layout->setWidget(0, QFormLayout::FieldRole, xpos);
  layout->setWidget(1, QFormLayout::FieldRole, ypos);
  layout->setWidget(2, QFormLayout::FieldRole, zpos);

  layout->setWidget(0, QFormLayout::LabelRole, xposSpinBox);
  layout->setWidget(1, QFormLayout::LabelRole, yposSpinBox);
  layout->setWidget(2, QFormLayout::LabelRole, zposSpinBox);

  connect(xposSpinBox, SIGNAL(valueChanged(int)),
	  this, SLOT(updateCamera()));
  connect(yposSpinBox, SIGNAL(valueChanged(int)),
	  this, SLOT(updateCamera()));
  connect(zposSpinBox, SIGNAL(valueChanged(int)),
	  this, SLOT(updateCamera()));

  QPushButton *button = new QPushButton("Print configuration");
  layout->setWidget(3, QFormLayout::SpanningRole, button);
  connect(button, SIGNAL(clicked()),
	  this, SLOT(printConfigurationClicked()));

  updateCamera();
}

void Controls::addLink(double lowerState, double upperState)
{
  QDoubleSpinBox *slider = new QDoubleSpinBox();
  slider->setMinimum(lowerState*100.0f);
  slider->setMaximum(upperState*100.0f);
  //  slider->setOrientation(Qt::Horizontal);
  slider->setValue(0);
  slider->setSingleStep(0.1);
  // Strangeness
  layout->addWidget(slider);
  layout->setWidget(layout->rowCount(), QFormLayout::SpanningRole, slider);   
  this->sliders.push_back(slider);
  QObject::connect(slider, SIGNAL(valueChanged(double)),
		   this, SLOT(updateState()));  
}


void Controls::updateCamera()
{
  camera->setPosition(xposSpinBox->value(),
                      yposSpinBox->value(),
		      zposSpinBox->value());
  emit cameraChanged();
}

void Controls::updateState()
{
  emit stateChanged();
}

MSLVector Controls::getState()
{
  MSLVector x(sliders.size());

  for (unsigned int i=0; i<sliders.size(); i++) {
    //    x[i] = sliders[i]->value()/100.0f;    
    x[i] = sliders[i]->value();    
  }

  return x;
}

void Controls::setState(MSLVector state)
{
  blockSignals(true);
  for (int i=0; i<state.dim(); i++) {
    //    sliders[i]->setValue(state[i]*100.0f);
    sliders[i]->setValue(state[i]);
  }
  blockSignals(false);
}

void Controls::printConfigurationClicked()
{
  emit printConfiguration();
}

#include "moc_controls.cpp"
