#include "glwidget.h"

GLWidget::GLWidget(QWidget *parent) : QGLWidget(parent)
{
  pos[0] = 0;
  pos[1] = 0;
  pos[2] = 0;
  rot[0] = 0;
  rot[1] = 0;
  rot[2] = 0;
  
  setMinimumSize(800, 600);
  makeCurrent();
  setAutoBufferSwap(false);
  setFocusPolicy(Qt::ClickFocus);
}


void GLWidget::initializeGL()
{
  glClearColor(1.0, 1.0, 1.0, 0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  float global_ambient[] = { 0.2f, 0.2f, 0.2f, 1.0f };
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
  float light0_ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
  glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
  float light0_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
  float light0_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_specular);
  float light0_position[] = {1.0, 0.0, 0.0, 800.0};
  glLightfv(GL_LIGHT0, GL_POSITION, light0_position);

  glEnable ( GL_LIGHTING );
  glEnable(GL_LIGHT0);

  glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE ) ;
  glEnable ( GL_COLOR_MATERIAL ) ;
 
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);  

  glFrontFace(GL_CW);
  //  glEnable(GL_CULL_FACE);
}

void GLWidget::resizeGL(int w, int h)
{
  width = w; height = h;
  glViewport(0, 0, w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(60.0f,(GLfloat)w/(GLfloat)h,1.0f,3000.0f);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void GLWidget::update()
{
  //  cout << "glwidget::update()"<<endl;
  QGLWidget::update();
  swapBuffers();
}

void GLWidget::paintGL()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  cam.use();

  drawCoordinateSystem();

  /*

  for (unsigned int i=0; i<links.size(); i++) {
    setTransform(links[i]->getPosX(), links[i]->getPosY(),links[i]->getPosZ(),
		 links[i]->getRotX(), links[i]->getRotY(),links[i]->getRotZ());   
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    //    useTransform();
    //    drawCoordinateSystem();

    //    if (i < 6)
    //      glColor3f(colors[i][0], colors[i][1], colors[i][2]);

    links[i]->draw();

    glPopMatrix();
  }*/
}

void GLWidget::drawCoordinateSystem()
{
  glPushMatrix();
  glLineWidth(2);
  glBegin(GL_LINES);
  glColor3f(1.0, 0.0, 0.0);
  glVertex3f(0.0, 0.0, 0.0);
  glVertex3f(200.0, 0.0, 0.0);
  glColor3f(0.0, 1.0, 0.0);
  glVertex3f(0.0, 0.0, 0.0);
  glVertex3f(0.0, 200.0, 0.0);
  glColor3f(0.0, 0.0, 1.0);
  glVertex3f(0.0, 0.0, 0.0);
  glVertex3f(0.0, 0.0, 200.0);
  glEnd();
  glPopMatrix();
}

void GLWidget::setTransform(float posx, float posy, float posz, float rotx, float roty, float rotz)
{
  pos[0] = posx;
  pos[1] = posy;
  pos[2] = posz;
  rot[0] = rotx;
  rot[1] = roty;
  rot[2] = rotz;
}

void GLWidget::useTransform()
{
  glTranslatef(pos[0], pos[1], pos[2]);

  glRotatef(rot[2]*180.0/PI, 0.0, 0.0, 1.0);
  glRotatef(rot[1]*180.0/PI, 0.0, 1.0, 0.0);
  glRotatef(rot[0]*180.0/PI, 1.0, 0.0, 0.0);
}

void GLWidget::addLink(Link *link)
{
  links.push_back(link);
}

void GLWidget::refresh()
{
  update();
}


void GLWidget::mousePressEvent(QMouseEvent *e)
{
  lastX = e->x();
  lastY = e->y();
}

void GLWidget::mouseMoveEvent(QMouseEvent *e)
{
  int diffX = e->x() - lastX;
  lastX = e->x();
  int diffY = e->y() - lastY;
  lastY = e->y();

  cam.Yaw((float)diffX/180.0f);
  cam.Tilt(-(float)diffY/180.0f);

  emit mustUpdate();
}

void GLWidget::keyReleaseEvent(QKeyEvent *e)
{
  if (e->key() == Qt::Key_W) {
    cam.move(30);
  } else if (e->key() == Qt::Key_S) {
    cam.move(-30);
  }

  emit mustUpdate();
}


void GLWidget::wheelEvent(QWheelEvent *e)
{
  cam.move(e->delta());

  emit mustUpdate();
}

#include "moc_glwidget.cpp"
