#include <math.h>
#include <fstream>
#include <MSL/defs.h>
#include "modelirbtwins.h"
#include "modelirb.h"
#include "dh.h"

#ifndef sqr
#define sqr(x) (x*x)
#endif
#ifndef PI
#define PI 3.14159265
#endif

ModelIRBTwins::ModelIRBTwins(string path) : ModelIRB(path)
{
  setDistance(320);
  setTolerance(10);
  setGamma(0.3);

  std::ifstream fin;

  StateDim1 = 6;
  StateDim2 = 6;

  StateDim = 12;

  READ_PARAMETER_OR_ERROR(NumBodies);
  this->NumBodies = 12;
  READ_PARAMETER_OR_ERROR(DH1);
  READ_PARAMETER_OR_ERROR(DH2);
  
  StateIndices1 = vector<int>(StateDim);
  StateIndices2 = vector<int>(StateDim);
  fin.open((FilePath + "StateIndices1").c_str());
  if (fin) {
    for (int i=0; i<StateDim1; i++) {
      fin >> StateIndices1[i];
    }
    fin.close();
  } else {
    cerr << "Could not find StateIndices1" << endl;
    exit(-1);
  }
  fin.open((FilePath + "StateIndices2").c_str());
  if (fin) {
    for (int i=0; i<StateDim2; i++) {
      fin >> StateIndices2[i];
    }
  } else {
    cerr << "Could not find StateIndices2" << endl;
    exit(-1);
    fin.close();
  }

  LowerState = MSLVector(StateDim);
  UpperState = MSLVector(StateDim);
  
  for (int i = 0; i < StateDim; i++)
    LowerState[i] = -1.0;
  for (int i = 0; i < StateDim; i++) 
    UpperState[i] = 1.0;

  READ_OPTIONAL_PARAMETER(LowerState);
  READ_OPTIONAL_PARAMETER(UpperState);

  MSLVector u;
  Inputs.clear();  // Otherwise its parent constructor will make some inputs
  // Add inputs that makes the robots move one at the time.
  for (int i = 0; i < StateDim; i++) {
    u = MSLVector(StateDim);
    u[i] = 0.01;
    Inputs.push_back(u);
    u = MSLVector(StateDim);
    u[i] = -0.01;
    Inputs.push_back(u);
  }

  // Add inputs that makes both robots move simultaneously.
  u = MSLVector(StateDim);
  u[0] = 0.01;
  u[6] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = -0.01;
  u[6] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = 0.01;
  u[6] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = -0.01;
  u[6] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = 0.01;
  u[7] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = -0.01;
  u[7] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = 0.01;
  u[7] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = -0.01;
  u[7] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = 0.01;
  u[8] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = -0.01;
  u[8] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = 0.01;
  u[8] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = -0.01;
  u[8] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[3] = 0.01;
  u[9] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[3] = -0.01;
  u[9] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[3] = 0.01;
  u[9] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[3] = -0.01;
  u[9] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[4] = 0.01;
  u[10] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[4] = -0.01;
  u[10] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[4] = 0.01;
  u[10] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[4] = -0.01;
  u[10] = 0.01;
  Inputs.push_back(u);

  u = MSLVector(StateDim);
  u[0] = 0.01;
  u[7] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = -0.01;
  u[7] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = 0.01;
  u[7] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = -0.01;
  u[7] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = 0.01;
  u[8] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = -0.01;
  u[8] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = 0.01;
  u[8] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[0] = -0.01;
  u[8] = 0.01;
  Inputs.push_back(u);
  
  u = MSLVector(StateDim);
  u[1] = 0.01;
  u[6] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = -0.01;
  u[6] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = 0.01;
  u[6] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[1] = -0.01;
  u[6] = 0.01;
  Inputs.push_back(u);

  u = MSLVector(StateDim);
  u[2] = 0.01;
  u[6] = 0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = -0.01;
  u[6] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = 0.01;
  u[6] = -0.01;
  Inputs.push_back(u);
  u = MSLVector(StateDim);
  u[2] = -0.01;
  u[6] = 0.01;
  Inputs.push_back(u);
}

/**
 * Calculate position and orientation of all links from the configuration vector.
 * Translate the base frame of the second robot 700mm.
 */
MSLVector ModelIRBTwins::StateToConfiguration(const MSLVector &x)
{
  MSLVector q;
  MSLVector A, Alpha, D, Theta;
  int i;
  MSLMatrix r(4,4), rn(4,4), ro(4,4);

  q = MSLVector(6*NumBodies);
  A = MSLVector(NumBodies);
  D = MSLVector(NumBodies);
  Alpha = MSLVector(NumBodies);
  Theta = MSLVector(NumBodies);

  for (i = 0; i < StateDim1; i++ ) {
    if (StateIndices1[i] != 0) {
      int y = StateIndices1[i];
      DH1[y-1] = x[i];
    }
  } 

  for (i = 0; i < StateDim2; i++ ) {
    if (StateIndices2[i] != 0) {
      int y = StateIndices2[i];
      DH2[y-1] = x[i+StateDim1];
    }
  } 
 
  int NumBodies = DH1.dim()/4;
  for (i = 0; i < NumBodies; i++) {
    Alpha[i] = DH1[i];
    Theta[i] = DH1[NumBodies*1+i];
    A[i] = DH1[NumBodies*2+i];
    D[i] = DH1[NumBodies*3+i];
  }

  DH::identity(r);
  
  for (i = 0; i < NumBodies; i++) {
    ro=r;
    rn = DH::T(A[i], Alpha[i], D[i], Theta[i]);
    DH::mul(r,ro,rn);
    DH::extract(q, 6*i, r);
  }


  NumBodies = DH2.dim()/4;
  for (i = 0; i < NumBodies; i++) {
    Alpha[i] = DH2[i];
    Theta[i] = DH2[NumBodies*1+i];
    A[i] = DH2[NumBodies*2+i];
    D[i] = DH2[NumBodies*3+i];
  }

  r = DH::T(700, 0, 0, 0);
  
  for (i = 0; i < NumBodies; i++) {
    ro=r;
    rn = DH::T(A[i], Alpha[i], D[i], Theta[i]);
    DH::mul(r,ro,rn);
    DH::extract(q, 6*(DH1.dim()/4)+6*i, r);
  }


  //  cout << q << endl;
  //  cout << "Dist: " << dist(q) << endl;
  //  cout << "Dist: " << dist(q) << " angle: " << angle(q) << endl;
  //  angle(q);
  return q;  
}

void ModelIRBTwins::setDistance(float distance)
{
  this->distance = distance;  
}

void ModelIRBTwins::setTolerance(float tolerance)
{
  this->tolerance = tolerance;
}

void ModelIRBTwins::setGamma(float gamma)
{
  this->gamma = gamma;
}

/**
 * Calculate distance between end effectors.
 */
double ModelIRBTwins::dist(const MSLVector &q)
{
  double dx = q[6*5] - q[6*11];
  double dy = q[6*5+1] - q[6*11+1];
  double dz = q[6*5+2] - q[6*11+2];

  double d = sqrt(sqr(dx)+sqr(dy)+sqr(dz));
  return d;
}

/**
 * Check whether the angle between the line through the end effectors
 * and the end effectors z axes is less than gamma.
 */
bool ModelIRBTwins::angle(const MSLVector &q)
{
  // Extract orientation for end effector of robot 1
  double psi1 = q[6*5+5];
  double theta1 = q[6*5+4];
  double phi1 = q[6*5+3];

  // Extract orientation for end effector of robot 2
  double psi2 = q[6*11+5];
  double theta2 = q[6*11+4];
  double phi2 = q[6*11+3];

  // Calculate z axis vector for end effector of robot 1
  MSLVector v1(sin(psi1)*sin(phi1)  + cos(psi1)*sin(theta1)*cos(phi1),
              -cos(psi1)*sin(phi1)  + sin(psi1)*sin(theta1)*cos(phi1),
	       cos(theta1)*cos(phi1));
  // Calculate z axis vector for end effector of robot 2
  MSLVector v2(sin(psi2)*sin(phi2)  + cos(psi2)*sin(theta2)*cos(phi2),
              -cos(psi2)*sin(phi2)  + sin(psi2)*sin(theta2)*cos(phi2),
	       cos(theta2)*cos(phi2));

  // Position of end effector of robot 1
  MSLVector p1(q[6*5],  q[6*5+1],  q[6*5+2]);
  // Position of end effector of robot 2
  MSLVector p2(q[6*11], q[6*11+1], q[6*11+2]);

  // Line through through both end effectors
  MSLVector f = p2 - p1;
  
  double angle1 = f.angle(v1);
  double angle2 = f.angle(v2);

  if (fabs(angle1) < gamma && fabs(PI-angle2) < gamma) {
    return true;
  }
  return false;
}


/**
 * Check that distance and angles between end effectors is within tolerances.
 */
bool ModelIRBTwins::Satisfied(const MSLVector &x)
{
  MSLVector q = StateToConfiguration(x);

  double d = dist(q);
  if (fabs(d-distance) > tolerance) {
    return false;
  }

  if (!angle(q)) {
    return false;
  }

  return true;
}
