#ifndef __PATH_H__
#define __PATH_H__

#include <QtGui>

#include <MSL/vector.h>
#include <MSL/problem.h>
#include <MSL/rrt.h>

#include "spline.h"
#include "splinerenderer.h"
#include "treerenderer.h"
#include "pathoptimization.h"

class Path : public QGroupBox {
  Q_OBJECT
  private:
    MSLVector &state;
    Problem *problem;
    RRTConCon *rrt;
    QFormLayout *layout;
    list<MSLVector>::iterator current;
    bool renderPathEnable;
    bool renderTreeEnable;
    vector<MSLVector> configurations;
    Spline spline;
    double deltaT;
    double t;
    SplineRenderer *splineRenderer;
    TreeRenderer *treeRenderer;
    PathOptimization *objective;
    QPushButton *satisfiedButton;
 public:
    Path(MSLVector &state, Problem *problem, GeomIRB *geom, QWidget *parent = 0);
    void render();
    void setRobot(ModelIRBSingle* irb);
    PathOptimization* getObjective();
 public slots:
    void calculatePath();
    void stepForward();
    void stepBackward();
    void renderPath();
    void renderTree();
    void optimizePath();
    void dividePath();
    void exportPath();
    void gotoInitialState();
    void gotoFinalState();
    void storePath();
    void restorePath();
 signals:
    void stateChanged();
};

#endif
