#ifndef __QUATERNION_H__
#define __QUATERNION_H__

#include <iostream>
#include <math.h>

using namespace std;

class Quaternion
{
 public:
  double a, b, c, d;
  Quaternion() { a=0; b=0; c=0; d=0;}
  Quaternion(double a, double b, double c, double d) 
    {this->a = a; this->b = b; this->c = c; this->d = d;}

  double norm() {
    return sqrt(a*a+b*b+c*c+d*d);
  }

  Quaternion operator*(Quaternion r) {
    return Quaternion(a*r.a - b*r.b - c*r.c - d*r.d,
		      a*r.b + b*r.a + c*r.d - d*r.c,
		      a*r.c - b*r.d + c*r.a + d*r.b,
		      a*r.d + b*r.c - c*r.b + d*r.a);
  }

  Quaternion operator/(double f) {
    if (f == 0)
      return *this;

    return Quaternion(a/f, b/f, c/f, d/f);
  }

  Quaternion conjugate() {
    return Quaternion(a, -b, -c, -d);
  }

  Quaternion rotate(Quaternion vector) {
    // Conjugation is equivalent to rotation
    Quaternion tmp = (*this) * vector;
    tmp = tmp * conjugate();
    return tmp;
  }

  friend ostream& operator<<(ostream& os, const Quaternion& q) {
    os << q.a << "," << q.b << "," << q.c << "," << q.d;
    return os;
  }

  static Quaternion axisAngle(double angle, double x, double y, double z) {
    double s = sin(angle*0.5);
    return Quaternion(cos(angle*0.5), s*x, s*y, s*z);		      
  }
};



#endif
