#include "spline.h"

Spline::Spline(unsigned int dimensions) : dimensions(dimensions)
{
}


void Spline::add(MSLVector p)
{
  points.push_back(p);
}

void Spline::shortcut(double t1, double t2)
{
  Vector new1(dimensions);
  Vector new2(dimensions);

  new1 = evaluate(t1);
  new2 = evaluate(t2);

  double t1m = t1 * points.size();
  int flr1 = floor(t1m);
  points.insert(points.begin()+flr1+1, new1);
  
  double t2m = t2 * points.size();
  int flr2 = floor(t2m);

  points.insert(points.begin()+flr2+1, new2);
    

  std::vector<int> positionsToRemove;

  for (unsigned int i = flr1+2; i<flr2+1; i++) {
      positionsToRemove.push_back(i-positionsToRemove.size());    
  }
  for (unsigned int i=0; i<positionsToRemove.size(); i++) {
    remove(positionsToRemove[i]);
  }
}

Vector Spline::evaluate(double t)
{
  if (points.size() == 0) {
    return NULL;
  }

  if (t<0) {
    return points[0];
  }

  double tm = t * points.size();
  double flr = floor(tm);
  if (flr+1 >= points.size()) {
    return points[flr];
  }
  double tm2 = tm - flr;
  
  Vector p(dimensions);

  p = points[flr] * (1.0-tm2) + points[flr+1] * tm2;

  return p;

}

void Spline::clear()
{
  points.clear();

}

unsigned int Spline::numPoints()
{
  return points.size();
}

void Spline::remove(unsigned int position)
{
  points.erase(points.begin() + position);
}

Vector Spline::getPoint(unsigned int i)
{
  return points[i];
}

void Spline::setPoint(unsigned int i, Vector v)
{
  points[i] = v;
}

Vector Spline::operator[](int i) const
{
  return points[i];
}

Vector& Spline::operator[](int i)
{
  return points[i];
}

void Spline::divide()
{
  Vector newPoint(dimensions);

  for (unsigned int i=0; i<numPoints()-1; i++) {
    newPoint = (points[i] + points[i+1])*0.5;
    points.insert(points.begin() + i+1, newPoint);
    i += 1;
  }
  cout << "numPoints() after divide(): " << numPoints() << endl;
}

void Spline::store()
{
  storedPoints = points;
}

void Spline::restore()
{
  points = storedPoints;
}

double Spline::length()
{
  double l = 0;

  MSLVector temp(dimensions);
  for (int i=0; i<points.size()-1; i++) {
    temp = points[i] - points[i+1];
    l += temp.length();
  }
  return l;
}
