#include "transform.h"

Transform::Transform(string str, Link *link, QWidget *parent) : QGroupBox(tr(str.c_str()), parent)
{
  this->link = link;

  layout = new QFormLayout(parent);
  this->setLayout(layout);

  transX = new QDoubleSpinBox;
  transX->setDecimals(1);
  transX->setMaximum(1000);
  transX->setMinimum(-1000);
  transX->setSuffix("mm");
  transX->setSingleStep(1);
  layout->setWidget(0, QFormLayout::FieldRole, transX);

  QLabel *xlabel = new QLabel("x");
  layout->setWidget(0,QFormLayout::LabelRole, xlabel);

  transY = new QDoubleSpinBox;
  transY->setDecimals(1);
  transY->setMaximum(1000);
  transY->setMinimum(-1000);
  transY->setSuffix("mm");
  transY->setSingleStep(1);
  layout->setWidget(1,QFormLayout::FieldRole, transY);

  QLabel *ylabel = new QLabel("y");
  layout->setWidget(1,QFormLayout::LabelRole, ylabel);

  transZ = new QDoubleSpinBox;
  transZ->setDecimals(1);
  transZ->setMaximum(1000);
  transZ->setMinimum(-1000);
  transZ->setSuffix("mm");
  transZ->setSingleStep(1);
  layout->setWidget(2,QFormLayout::FieldRole, transZ);

  QLabel *zlabel = new QLabel("z");
  layout->setWidget(2,QFormLayout::LabelRole, zlabel);
  
  QPushButton *button = new QPushButton("Apply translation");
  layout->setWidget(3,QFormLayout::FieldRole, button);
  connect(button, SIGNAL(clicked()),
	  this, SLOT(applyTransformation()));

  rotX = new QDoubleSpinBox;
  rotX->setDecimals(2);
  rotX->setMaximum(180);
  rotX->setMinimum(-180);
  rotX->setSuffix(" deg");
  rotX->setSingleStep(1);
  layout->setWidget(4,QFormLayout::FieldRole, rotX);

  xlabel = new QLabel("x");
  layout->setWidget(4,QFormLayout::LabelRole, xlabel);

  rotY = new QDoubleSpinBox;
  rotY->setDecimals(2);
  rotY->setMaximum(180);
  rotY->setMinimum(-180);
  rotY->setSuffix(" deg");
  rotY->setSingleStep(1);
  layout->setWidget(5,QFormLayout::FieldRole, rotY);

  ylabel = new QLabel("y");
  layout->setWidget(5,QFormLayout::LabelRole, ylabel);

  rotZ = new QDoubleSpinBox;
  rotZ->setDecimals(2);
  rotZ->setMaximum(180);
  rotZ->setMinimum(-180);
  rotZ->setSuffix(" deg");
  rotZ->setSingleStep(1);
  layout->setWidget(6,QFormLayout::FieldRole, rotZ);

  zlabel = new QLabel("z");
  layout->setWidget(6,QFormLayout::LabelRole, zlabel);

  button = new QPushButton("Apply rotation");
  layout->setWidget(7,QFormLayout::FieldRole, button);
  connect(button, SIGNAL(clicked()),
	  this, SLOT(applyRotation()));

  
  scaleX = new QDoubleSpinBox;
  scaleX->setDecimals(2);
  scaleX->setMaximum(180);
  scaleX->setMinimum(0.01);
  scaleX->setSingleStep(1);
  scaleX->setValue(1.0);
  layout->setWidget(8,QFormLayout::FieldRole, scaleX);

  xlabel = new QLabel("x");
  layout->setWidget(8,QFormLayout::LabelRole, xlabel);

  scaleY = new QDoubleSpinBox;
  scaleY->setDecimals(2);
  scaleY->setMaximum(180);
  scaleY->setMinimum(0.01);
  scaleY->setSingleStep(1);
  scaleY->setValue(1.0);
  layout->setWidget(9,QFormLayout::FieldRole, scaleY);

  ylabel = new QLabel("y");
  layout->setWidget(9,QFormLayout::LabelRole, ylabel);

  scaleZ = new QDoubleSpinBox;
  scaleZ->setDecimals(2);
  scaleZ->setMaximum(180);
  scaleZ->setMinimum(0.01);
  scaleZ->setSingleStep(1);
  scaleZ->setValue(1.0);
  layout->setWidget(10,QFormLayout::FieldRole, scaleZ);

  zlabel = new QLabel("z");
  layout->setWidget(10,QFormLayout::LabelRole, zlabel);

  button = new QPushButton("Apply scaling");
  layout->setWidget(11,QFormLayout::FieldRole, button);
  connect(button, SIGNAL(clicked()),
	  this, SLOT(applyScaling()));


  button = new QPushButton("Switch direction");
  layout->setWidget(12, QFormLayout::FieldRole, button);
  connect(button, SIGNAL(clicked()),
	  this, SLOT(switchDirection()));

  button = new QPushButton("Save");
  layout->setWidget(13, QFormLayout::FieldRole, button);
  connect(button, SIGNAL(clicked()),
	  this, SLOT(save()));
}

void Transform::applyTransformation()
{
  double tx = transX->value();
  double ty = transY->value();
  double tz = transZ->value();

  transX->setValue(0.0);
  transY->setValue(0.0);
  transZ->setValue(0.0);

  link->translate(tx, ty, tz);

  emit update();
}

void Transform::applyRotation()
{
  double rx = rotZ->value();
  double ry = rotY->value();
  double rz = rotX->value();

  rotX->setValue(0.0);
  rotY->setValue(0.0);
  rotZ->setValue(0.0);

  link->rotate(rx, ry, rz);

  emit update();
}

void Transform::applyScaling()
{
  double sx = scaleZ->value();
  double sy = scaleY->value();
  double sz = scaleX->value();

  scaleX->setValue(1.0);
  scaleY->setValue(1.0);
  scaleZ->setValue(1.0);

  link->scale(sx,sy,sz);  

  emit update();
}

void Transform::switchDirection()
{
  link->switchDirection();
  
  emit update();
}

void Transform::save()
{
  link->save();
}


#include "moc_transform.cpp"
