function collision = collision_check(pointA, pointB, geometry)
    % Check that pointA and pointB are on the same side of the lines in
    % geometry.

    for edge = geometry.edges
        edge = edge{1};
        linePointA = geometry.vertices(edge(1),:);
        linePointB = geometry.vertices(edge(2),:);
        if line_collision(pointA, pointB, linePointA, linePointB)
            collision = false;
            return;
        end
    end    

    collision = true;    
end

function c = line_collision(pointA, pointB, linePointA, linePointB)
    lineVector = linePointA - linePointB;
    length = norm(lineVector);
    if length < 0.00001
        c = false;
        return;
    end
        
    lineVector = lineVector / length;
    
    % First check whether the points are on the same side
    
    temp = lineVector(1);
    lineVector(1) = -lineVector(2);
    lineVector(2) = temp;
    
    vectorA = pointA - linePointA;
    vectorB = pointB - linePointB;
    
    signA = dot(lineVector, vectorA);
    signB = dot(lineVector, vectorB);
    
    if signA*signB > 0
        c = false;
        return;
    end
       
    % Second find out where on the line they cross
%     text(pointA(1), pointA(2), 'A');
%     text(pointB(1), pointB(2), 'B');
%     text(linePointA(1), linePointA(2), 'A');
%     text(linePointB(1), linePointB(2), 'B');    
%     plot([pointA(1) pointB(1)], [pointA(2) pointB(2)])
%     plot([linePointA(1) linePointB(1)], [linePointA(2) linePointB(2)], 'c')

    l = linePointA - linePointB;
    ll = norm(l);
    l = l';
    l = l/norm(l);
    v = pointA-pointB;
    vv = norm(v);
    v = v/norm(v);
    v = v';
    A = [v,l];
    b = (pointB - linePointB)';
    
    x = A\b;
    t = x(1);
    s = x(2);
    
    if s > 0 && s < ll
        c = true;
    else
        c = false;    
    end
  
end