% Extend the tree with a new vertex
function r = rrt_extend(rrt, v, delta, collision_fnc, geometry)
    [index, edge_point, edge_index] = find_closest(rrt, v);
    
    if isnan(index)
%        fprintf('Closest is edge\n');
        delta = min(delta, norm(v-edge_point));
        vnew = (v - edge_point)/norm(v-edge_point)*delta+edge_point;
        if nargin == 5
            if collision_fnc(edge_point, vnew, geometry) == false
                r = rrt;
                return;
            end
        end
        rrt.vertices = [rrt.vertices; edge_point; vnew];
        old_edge     = rrt.edges{edge_index};
        rrt.edges(edge_index) = [];
        rrt.edges    = [rrt.edges, [old_edge(1), length(rrt.vertices)-1]];
        rrt.edges    = [rrt.edges, [length(rrt.vertices)-1,old_edge(2)]];
        rrt.edges    = [rrt.edges, [length(rrt.vertices)-1,length(rrt.vertices)]];
        rrt.parent_of_vertex = [rrt.parent_of_vertex, old_edge(1)];
        rrt.parent_of_vertex = [rrt.parent_of_vertex, length(rrt.vertices)-1];
    else
%        fprintf('Closest is vertex\n');
        delta = min(delta, norm(v-rrt.vertices(index,:)));
        vnew = delta*(v-rrt.vertices(index,:))/norm(v-rrt.vertices(index,:))+rrt.vertices(index,:);
        if nargin == 5
            if collision_fnc(rrt.vertices(index,:), vnew, geometry) == false
                r = rrt;                
                return;
            end
        end
        rrt.vertices = [rrt.vertices; vnew];
        rrt.edges    = [rrt.edges, [index, length(rrt.vertices)]];
        rrt.parent_of_vertex = [rrt.parent_of_vertex, index];
    end

            
    if strcmp(rrt.status, 'unsolved') && norm(vnew-rrt.goal_point) < delta
            rrt.status = 'solved';
            r = rrt_extend(rrt, rrt.goal_point, delta, collision_fnc, geometry);
    else
        r = rrt;
    end
end

function [index, edge_point, edge_index] = find_closest(rrt, v)
    vindex = 1;
    vdist = Inf;
    % First find the closest vertex
    for i = 1:size(rrt.vertices,1)
        point = rrt.vertices(i,:);
        d = norm(v-point);
        if (d < vdist)
            vdist  = d;
            vindex = i;
        end
    end
    
    eindex = 1;
    edist  = Inf;
    epoint = [Inf, Inf];
    % Second find the closest point on edge
    for i = 1:size(rrt.edges, 2)
        point = find_closest_edge_point(rrt, i, v);
        d = norm(v-point);
        if (d < edist)
            edist  = d;
            eindex = i;
            epoint = point;
        end
    end
    
    if (vdist <= edist)
%    if true
        index      = vindex;
        edge_point = [Inf, Inf];
        edge_index = NaN;
    else
        index      = NaN;
        edge_point = epoint;
        edge_index = eindex;
    end
end

function [p] = find_closest_edge_point(rrt, edge_index, v)
    edge = rrt.edges{edge_index};
    pa   = rrt.vertices(edge(1), :);
    pb   = rrt.vertices(edge(2), :);
    
    edge_vector  = pb - pa;
    evlength = norm(edge_vector);
    if (evlength <= 0)
        p = [Inf, Inf];
        return;
    end
    edge_vector = edge_vector / evlength;
    
    point_vector = v - pa;
    pvlength = norm(point_vector);
    if (pvlength <= 0)
        p = [Inf, Inf];
        return;
    end
    point_vector = point_vector;
    
    dist = dot(point_vector, edge_vector);
    if (abs(dist) >= evlength || dist <= 0)
        p = [Inf, Inf];
        return;
    end
    
    p = pa + edge_vector*dist;
end